[CCCL] Add missing CCCL components: c2h, nvbench_helper, cmake, cudax, AGENTS.md
Added 863 files from NVIDIA/cccl sparse checkout: - c2h/ (27 files): Catch2 test helpers — generators, validators, runner - nvbench_helper/ (10 files): Benchmark harness utilities - cmake/ (29 files): CMake presets and build helpers - cudax/ (794 files): Experimental CUDA extensions - AGENTS.md: NVIDIA's official AI agent instructions for CCCL - CMakePresets.json: Standardized build configurations - cccl-version.json: Version tracking Also added CCCL_ASSET_MAP.md mapping all 4295 CCCL files to competition value and PRD items. cccl_upstream now covers 100% of competition-critical assets: - 27 tuning headers (SM80/90/100 benchmark data) - 32 dispatch headers (algorithm implementations) - 60 Thrust examples (correctness verification) - 217 CUB Catch2 tests (regression matrix) - 153 CUB benchmarks (parameter space search) - 18 CUB examples (API verification) - 27 test helpers + benchmark harness - 794 cudax experimental extensions
This commit is contained in:
212
cccl_upstream/cudax/test/group/mapping/take.cu
Normal file
212
cccl_upstream/cudax/test/group/mapping/take.cu
Normal file
@@ -0,0 +1,212 @@
|
||||
//===----------------------------------------------------------------------===//
|
||||
//
|
||||
// Part of CUDA Experimental in CUDA C++ Core Libraries,
|
||||
// under the Apache License v2.0 with LLVM Exceptions.
|
||||
// See https://llvm.org/LICENSE.txt for license information.
|
||||
// SPDX-License-Identifier: Apache-2.0 WITH LLVM-exception
|
||||
// SPDX-FileCopyrightText: Copyright (c) 2026 NVIDIA CORPORATION & AFFILIATES.
|
||||
//
|
||||
//===----------------------------------------------------------------------===//
|
||||
|
||||
#include <cuda/devices>
|
||||
#include <cuda/hierarchy>
|
||||
#include <cuda/launch>
|
||||
#include <cuda/std/cstddef>
|
||||
#include <cuda/std/numeric>
|
||||
#include <cuda/std/type_traits>
|
||||
#include <cuda/std/utility>
|
||||
#include <cuda/stream>
|
||||
#include <cuda/warp>
|
||||
|
||||
#include <cuda/experimental/group.cuh>
|
||||
|
||||
#include "group_testing.cuh"
|
||||
|
||||
namespace
|
||||
{
|
||||
template <cuda::std::size_t N, class Config>
|
||||
__device__ void test_take(Config config)
|
||||
{
|
||||
constexpr auto n = static_cast<unsigned>(N);
|
||||
|
||||
// Test static N.
|
||||
{
|
||||
using Mapping = cudax::take<N>;
|
||||
|
||||
// Test default constructor.
|
||||
{
|
||||
static_assert(cuda::std::is_trivially_default_constructible_v<Mapping>);
|
||||
static_assert(cuda::std::is_empty_v<Mapping>);
|
||||
|
||||
Mapping mapping;
|
||||
CHECK(mapping.unit_count() == n);
|
||||
}
|
||||
|
||||
// Test static_unit_count().
|
||||
{
|
||||
static_assert(cuda::std::is_same_v<cuda::std::size_t, decltype(Mapping::static_unit_count())>);
|
||||
static_assert(noexcept(Mapping::static_unit_count()));
|
||||
|
||||
static_assert(Mapping::static_unit_count() == N);
|
||||
}
|
||||
|
||||
// Test unit_count().
|
||||
{
|
||||
static_assert(cuda::std::is_same_v<unsigned, decltype(cuda::std::declval<const Mapping>().unit_count())>);
|
||||
static_assert(noexcept(cuda::std::declval<const Mapping>().unit_count()));
|
||||
|
||||
const Mapping mapping;
|
||||
CHECK(mapping.unit_count() == n);
|
||||
}
|
||||
|
||||
// Test map(...).
|
||||
{
|
||||
const cudax::this_warp parent_group{config};
|
||||
const ThreadsInWarpMappingResult prev_mapping_result;
|
||||
|
||||
static_assert(cudax::__group_mapping_result<decltype(cuda::std::declval<const Mapping>().map(
|
||||
cuda::gpu_thread, parent_group, prev_mapping_result))>);
|
||||
static_assert(
|
||||
noexcept(cuda::std::declval<const Mapping>().map(cuda::gpu_thread, parent_group, prev_mapping_result)));
|
||||
|
||||
const Mapping mapping;
|
||||
auto result = mapping.map(cuda::gpu_thread, parent_group, prev_mapping_result);
|
||||
using Result = decltype(result);
|
||||
|
||||
static_assert(Result::static_group_count() == ThreadsInWarpMappingResult::static_group_count());
|
||||
static_assert(Result::static_unit_count() == N);
|
||||
static_assert(Result::is_always_exhaustive()
|
||||
== (Result::static_unit_count() == ThreadsInWarpMappingResult::static_unit_count()));
|
||||
static_assert(Result::is_always_contiguous());
|
||||
|
||||
const auto is_valid_ref = cuda::std::cmp_less(cuda::gpu_thread.rank(cuda::warp), n);
|
||||
CHECK(result.is_valid() == is_valid_ref);
|
||||
|
||||
if (is_valid_ref)
|
||||
{
|
||||
CHECK(result.group_count() == prev_mapping_result.group_count());
|
||||
CHECK(result.group_rank() == prev_mapping_result.group_rank());
|
||||
|
||||
CHECK(result.unit_count() == n);
|
||||
CHECK(result.unit_rank() == cuda::gpu_thread.rank(cuda::warp));
|
||||
|
||||
const auto lane_mask_ref = ((N < 32) ? ((1u << N) - 1) : ~0u);
|
||||
CHECK(result.lane_mask() == cuda::device::lane_mask{lane_mask_ref});
|
||||
|
||||
CHECK(result.is_valid());
|
||||
}
|
||||
}
|
||||
}
|
||||
|
||||
// Test dynamic Ns.
|
||||
{
|
||||
using Mapping = cudax::take<cuda::std::dynamic_extent>;
|
||||
|
||||
// Test default constructor.
|
||||
{
|
||||
static_assert(cuda::std::is_nothrow_default_constructible_v<Mapping>);
|
||||
|
||||
Mapping mapping;
|
||||
CHECK(mapping.unit_count() == 0);
|
||||
}
|
||||
|
||||
// Test the mapping is constructible from n.
|
||||
{
|
||||
static_assert(cuda::std::is_nothrow_constructible_v<Mapping, unsigned>);
|
||||
|
||||
cudax::take mapping{n};
|
||||
static_assert(cuda::std::is_same_v<decltype(mapping), Mapping>);
|
||||
CHECK(mapping.unit_count() == n);
|
||||
}
|
||||
|
||||
// Test static_unit_count().
|
||||
{
|
||||
static_assert(cuda::std::is_same_v<cuda::std::size_t, decltype(Mapping::static_unit_count())>);
|
||||
static_assert(noexcept(Mapping::static_unit_count()));
|
||||
|
||||
static_assert(Mapping::static_unit_count() == cuda::std::dynamic_extent);
|
||||
}
|
||||
|
||||
// Test unit_count().
|
||||
{
|
||||
static_assert(cuda::std::is_same_v<unsigned, decltype(cuda::std::declval<const Mapping>().unit_count())>);
|
||||
static_assert(noexcept(cuda::std::declval<const Mapping>().unit_count()));
|
||||
|
||||
const Mapping mapping{n};
|
||||
CHECK(mapping.unit_count() == n);
|
||||
}
|
||||
|
||||
// Test map(...).
|
||||
{
|
||||
const cudax::this_warp parent_group{config};
|
||||
const ThreadsInWarpMappingResult prev_mapping_result;
|
||||
|
||||
static_assert(cudax::__group_mapping_result<decltype(cuda::std::declval<const Mapping>().map(
|
||||
cuda::gpu_thread, parent_group, prev_mapping_result))>);
|
||||
static_assert(
|
||||
noexcept(cuda::std::declval<const Mapping>().map(cuda::gpu_thread, parent_group, prev_mapping_result)));
|
||||
|
||||
const Mapping mapping{n};
|
||||
auto result = mapping.map(cuda::gpu_thread, parent_group, prev_mapping_result);
|
||||
using Result = decltype(result);
|
||||
|
||||
static_assert(Result::static_group_count() == ThreadsInWarpMappingResult::static_group_count());
|
||||
static_assert(Result::static_unit_count() == cuda::std::dynamic_extent);
|
||||
static_assert(!Result::is_always_exhaustive());
|
||||
static_assert(Result::is_always_contiguous());
|
||||
|
||||
const auto is_valid_ref = cuda::std::cmp_less(cuda::gpu_thread.rank(cuda::warp), n);
|
||||
CHECK(result.is_valid() == is_valid_ref);
|
||||
|
||||
if (is_valid_ref)
|
||||
{
|
||||
CHECK(result.group_count() == prev_mapping_result.group_count());
|
||||
CHECK(result.group_rank() == prev_mapping_result.group_rank());
|
||||
|
||||
CHECK(result.unit_count() == n);
|
||||
CHECK(result.unit_rank() == cuda::gpu_thread.rank(cuda::warp));
|
||||
|
||||
const auto lane_mask_ref = ((N < 32) ? ((1u << N) - 1) : ~0u);
|
||||
CHECK(result.lane_mask() == cuda::device::lane_mask{lane_mask_ref});
|
||||
|
||||
CHECK(result.is_valid());
|
||||
}
|
||||
}
|
||||
}
|
||||
}
|
||||
|
||||
struct TestKernel
|
||||
{
|
||||
template <class Config>
|
||||
__device__ void operator()(const Config& config)
|
||||
{
|
||||
test_take<0>(config);
|
||||
test_take<1>(config);
|
||||
test_take<2>(config);
|
||||
test_take<3>(config);
|
||||
test_take<4>(config);
|
||||
test_take<14>(config);
|
||||
test_take<16>(config);
|
||||
test_take<30>(config);
|
||||
test_take<32>(config);
|
||||
}
|
||||
};
|
||||
} // namespace
|
||||
|
||||
C2H_TEST("Take mapping", "[group]")
|
||||
{
|
||||
const auto device = cuda::devices[0];
|
||||
|
||||
const cuda::stream stream{device};
|
||||
|
||||
{
|
||||
const auto config = cuda::make_config(cuda::grid_dims<1>(), cuda::block_dims<8, 4>());
|
||||
cuda::launch(stream, config, TestKernel{});
|
||||
}
|
||||
{
|
||||
const auto config = cuda::make_config(cuda::grid_dims<1>(), cuda::block_dims(dim3{8, 4}));
|
||||
cuda::launch(stream, config, TestKernel{});
|
||||
}
|
||||
|
||||
stream.sync();
|
||||
}
|
||||
Reference in New Issue
Block a user